DEFAULT_AUTH_PATH = "/api/rest/v1/auth"
DEFAULT_BASE_PATH = "/api/json/v1"

DEFAULT_TIMEOUT = 30000
DEFAULT_AUTH_HEADER = "X-Cassandra-Token"
DEFAULT_KEYSPACE_NAME = "default_keyspace"
DEFAULT_REGION = "us-east1"

DEFAULT_DEV_OPS_PATH_PREFIX = "/v2"
DEFAULT_DEV_OPS_URL = "api.astra.datastax.com"



import requests
import logging
import re

from astrapy import __version__
from astrapy.defaults import DEFAULT_TIMEOUT


logger = logging.getLogger(__name__)
# logger.setLevel(logging.DEBUG) # Apply if wishing to debug requests


class http_methods:
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"


package_name = __name__.split(".")[0]


def log_request_response(r, json_data):
    logger.debug(f"Request URL: {r.url}")
    logger.debug(f"Request method: {r.request.method}")
    logger.debug(f"Request headers: {r.request.headers}")

    if json_data:
        logger.debug(f"Request payload: {json_data}")

    logger.debug(f"Response status code: {r.status_code}")
    logger.debug(f"Response headers: {r.headers}")
    logger.debug(f"Response content: {r.text}")


def make_request(
    base_url,
    auth_header,
    token,
    method=http_methods.POST,
    path=None,
    json_data=None,
    url_params=None,
):
    r = requests.request(
        method=method,
        url=f"{base_url}{path}",
        params=url_params,
        json=json_data,
        timeout=DEFAULT_TIMEOUT,
        headers={auth_header: token, "User-Agent": f"{package_name}/{__version__}"},
    )

    if logger.isEnabledFor(logging.DEBUG):
        log_request_response(r, json_data)

    return r


def make_payload(top_level, **kwargs):
    params = {}
    for key, value in kwargs.items():
        params[key] = value

    json_query = {top_level: {}}

    # Adding keys only if they're provided
    for key, value in params.items():
        if value is not None:
            json_query[top_level][key] = value

    return json_query


def parse_endpoint_url(url):
    # Regular expression pattern to match the given URL format
    pattern = r"https://(?P<db_id>[a-fA-F0-9\-]{36})-(?P<db_region>[a-zA-Z0-9\-]+)\.(?P<db_hostname>[a-zA-Z0-9\-\.]+\.com)"

    match = re.match(pattern, url)
    if match:
        return (
            match.group("db_id"),
            match.group("db_region"),
            match.group("db_hostname"),
        )
    else:
        raise ValueError("Invalid URL format")



# Copyright DataStax, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from astrapy.utils import make_request, http_methods
from astrapy.defaults import DEFAULT_DEV_OPS_PATH_PREFIX, DEFAULT_DEV_OPS_URL

import logging

logger = logging.getLogger(__name__)


class AstraDBOps:
    def __init__(self, token, dev_ops_url=None):
        dev_ops_url = dev_ops_url or DEFAULT_DEV_OPS_URL

        self.token = "Bearer " + token
        self.base_url = f"https://{dev_ops_url}{DEFAULT_DEV_OPS_PATH_PREFIX}"

    def _ops_request(self, method, path, options=None, json_data=None):
        options = {} if options is None else options

        return make_request(
            base_url=self.base_url,
            method=method,
            auth_header="Authorization",
            token=self.token,
            json_data=json_data,
            url_params=options,
            path=path,
        )

    def get_databases(self, options=None):
        response = self._ops_request(
            method=http_methods.GET, path="/databases", options=options
        ).json()

        return response

    def create_database(self, database_definition=None):
        r = self._ops_request(
            method=http_methods.POST, path="/databases", json_data=database_definition
        )

        if r.status_code == 201:
            return {"id": r.headers["Location"]}

        return None

    def terminate_database(self, database=""):
        r = self._ops_request(
            method=http_methods.POST, path=f"/databases/{database}/terminate"
        )

        if r.status_code == 202:
            return database

        return None

    def get_database(self, database="", options=None):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/databases/{database}",
            options=options,
        ).json()

    def create_keyspace(self, database="", keyspace=""):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/keyspaces/{keyspace}",
        )

    def park_database(self, database=""):
        return self._ops_request(
            method=http_methods.POST, path=f"/databases/{database}/park"
        ).json()

    def unpark_database(self, database=""):
        return self._ops_request(
            method=http_methods.POST, path=f"/databases/{database}/unpark"
        ).json()

    def resize_database(self, database="", options=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/resize",
            json_data=options,
        ).json()

    def reset_database_password(self, database="", options=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/resetPassword",
            json_data=options,
        ).json()

    def get_secure_bundle(self, database=""):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/secureBundleURL",
        ).json()

    def get_datacenters(self, database=""):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/databases/{database}/datacenters",
        ).json()

    def create_datacenter(self, database="", options=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/datacenters",
            json_data=options,
        ).json()

    def terminate_datacenter(self, database="", datacenter=""):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/datacenters/{datacenter}/terminate",
        ).json()

    def get_access_list(self, database=""):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/databases/{database}/access-list",
        ).json()

    def replace_access_list(self, database="", access_list=None):
        return self._ops_request(
            method=http_methods.PUT,
            path=f"/databases/{database}/access-list",
            json_data=access_list,
        ).json()

    def update_access_list(self, database="", access_list=None):
        return self._ops_request(
            method=http_methods.PATCH,
            path=f"/databases/{database}/access-list",
            json_data=access_list,
        ).json()

    def add_access_list_address(self, database="", address=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/databases/{database}/access-list",
            json_data=address,
        ).json()

    def delete_access_list(self, database=""):
        return self._ops_request(
            method=http_methods.DELETE,
            path=f"/databases/{database}/access-list",
        ).json()

    def get_private_link(self, database=""):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/organizations/clusters/{database}/private-link",
        ).json()

    def get_datacenter_private_link(self, database="", datacenter=""):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/organizations/clusters/{database}/datacenters/{datacenter}/private-link",
        ).json()

    def create_datacenter_private_link(
        self, database="", datacenter="", private_link=None
    ):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/organizations/clusters/{database}/datacenters/{datacenter}/private-link",
            json_data=private_link,
        ).json()

    def create_datacenter_endpoint(self, database="", datacenter="", endpoint=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/organizations/clusters/{database}/datacenters/{datacenter}/endpoint",
            json_data=endpoint,
        ).json()

    def update_datacenter_endpoint(self, database="", datacenter="", endpoint=None):
        return self._ops_request(
            method=http_methods.PUT,
            path=f"/organizations/clusters/{database}/datacenters/{datacenter}/endpoints/{endpoint['id']}",
            json_data=endpoint,
        ).json()

    def get_datacenter_endpoint(self, database="", datacenter="", endpoint=""):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/organizations/clusters/{database}/datacenters/{datacenter}/endpoints/{endpoint}",
        ).json()

    def delete_datacenter_endpoint(self, database="", datacenter="", endpoint=""):
        return self._ops_request(
            method=http_methods.DELETE,
            path=f"/organizations/clusters/{database}/datacenters/{datacenter}/endpoints/{endpoint}",
        ).json()

    def get_available_classic_regions(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/availableRegions"
        ).json()

    def get_available_regions(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/regions/serverless"
        ).json()

    def get_roles(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/organizations/roles"
        ).json()

    def create_role(self, role_definition=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/organizations/roles",
            json_data=role_definition,
        ).json()

    def get_role(self, role=""):
        return self._ops_request(
            method=http_methods.GET, path=f"/organizations/roles/{role}"
        ).json()

    def update_role(self, role="", role_definition=None):
        return self._ops_request(
            method=http_methods.PUT,
            path=f"/organizations/roles/{role}",
            json_data=role_definition,
        ).json()

    def delete_role(self, role=""):
        return self._ops_request(
            method=http_methods.DELETE, path=f"/organizations/roles/{role}"
        ).json()

    def invite_user(self, user_definition=None):
        return self._ops_request(
            method=http_methods.PUT,
            path=f"/organizations/users",
            json_data=user_definition,
        ).json()

    def get_users(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/organizations/users"
        ).json()

    def get_user(self, user=""):
        return self._ops_request(
            method=http_methods.GET, path=f"/organizations/users/{user}"
        ).json()

    def remove_user(self, user=""):
        return self._ops_request(
            method=http_methods.DELETE, path=f"/organizations/users/{user}"
        ).json()

    def update_user_roles(self, user="", roles=None):
        return self._ops_request(
            method=http_methods.PUT,
            path=f"/organizations/users/{user}/roles",
            json_data=roles,
        ).json()

    def get_clients(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/clientIdSecrets"
        ).json()

    def create_token(self, roles=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/clientIdSecrets",
            json_data=roles,
        ).json()

    def delete_token(self, token=""):
        return self._ops_request(
            method=http_methods.DELETE, path=f"/clientIdSecret/{token}"
        ).json()

    def get_organization(self):
        return self._ops_request(method=http_methods.GET, path=f"/currentOrg").json()

    def get_access_lists(self):
        return self._ops_request(method=http_methods.GET, path=f"/access-lists").json()

    def get_access_list_template(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/access-list/template"
        ).json()

    def validate_access_list(self):
        return self._ops_request(
            method=http_methods.POST, path=f"/access-list/validate"
        ).json()

    def get_private_links(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/organizations/private-link"
        ).json()

    def get_streaming_providers(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/streaming/providers"
        ).json()

    def get_streaming_tenants(self):
        return self._ops_request(
            method=http_methods.GET, path=f"/streaming/tenants"
        ).json()

    def create_streaming_tenant(self, tenant=None):
        return self._ops_request(
            method=http_methods.POST,
            path=f"/streaming/tenants",
            json_data=tenant,
        ).json()

    def delete_streaming_tenant(self, tenant="", cluster=""):
        return self._ops_request(
            method=http_methods.DELETE,
            path=f"/streaming/tenants/{tenant}/clusters/{cluster}",
            json_data=tenant,
        ).json()

    def get_streaming_tenant(self, tenant=""):
        return self._ops_request(
            method=http_methods.GET,
            path=f"/streaming/tenants/{tenant}/limits",
        ).json()



# Copyright DataStax, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from astrapy.defaults import (
    DEFAULT_AUTH_HEADER,
    DEFAULT_KEYSPACE_NAME,
    DEFAULT_BASE_PATH,
)
from astrapy.utils import make_payload, make_request, http_methods, parse_endpoint_url

import logging
import json

logger = logging.getLogger(__name__)


class AstraDBCollection:
    def __init__(
        self,
        collection_name,
        astra_db=None,
        token=None,
        api_endpoint=None,
        namespace=None,
    ):
        if astra_db is None:
            if token is None or api_endpoint is None:
                raise AssertionError("Must provide token and api_endpoint")

            astra_db = AstraDB(
                token=token, api_endpoint=api_endpoint, namespace=namespace
            )

        self.astra_db = astra_db
        self.collection_name = collection_name
        self.base_path = f"{self.astra_db.base_path}/{collection_name}"

    def _request(self, *args, skip_error_check=False, **kwargs):
        response = make_request(
            *args,
            **kwargs,
            base_url=self.astra_db.base_url,
            auth_header=DEFAULT_AUTH_HEADER,
            token=self.astra_db.token,
        )
        responsebody = response.json()

        if not skip_error_check and "errors" in responsebody:
            raise ValueError(json.dumps(responsebody["errors"]))
        else:
            return responsebody

    def _get(self, path=None, options=None):
        full_path = f"{self.base_path}/{path}" if path else self.base_path
        response = self._request(
            method=http_methods.GET, path=full_path, url_params=options
        )
        if isinstance(response, dict):
            return response
        return None

    def _post(self, path=None, document=None):
        response = self._request(
            method=http_methods.POST, path=f"{self.base_path}", json_data=document
        )
        return response

    def _pre_process_find(self, vector, fields=None):
        # Must pass a vector
        if not vector:
            return ValueError("Must pass a vector")

        # Edge case for field selection
        if fields and "$similarity" in fields:
            raise ValueError("Please use the `include_similarity` parameter")

        # Build the new vector parameter
        sort = {"$vector": vector}

        # Build the new fields parameter
        # Note: do not leave projection={}, make it None
        # (or it will devour $similarity away in the API response)
        if fields is not None and len(fields) > 0:
            projection = {f: 1 for f in fields}
        else:
            projection = None

        return sort, projection

    def _finalize_find_return(self, itm, include_similarity=True):
        # Pop the returned similarity score
        if "$similarity" in itm:
            similarity = itm.pop("$similarity")
            if include_similarity:
                itm["$similarity"] = similarity

        return itm

    def _post_process_find(
        self,
        raw_find_result,
        include_similarity=True,
        _key="documents",
    ):
        if isinstance(raw_find_result["data"][_key], list):
            final_result = [
                self._finalize_find_return(
                    itm,
                    include_similarity=include_similarity,
                )
                for itm in raw_find_result["data"][_key]
            ]
        else:
            final_result = self._finalize_find_return(
                raw_find_result["data"][_key],
                include_similarity=include_similarity,
            )

        return final_result

    def get(self, path=None):
        return self._get(path=path)

    def find(self, filter=None, projection=None, sort={}, options=None):
        json_query = make_payload(
            top_level="find",
            filter=filter,
            projection=projection,
            options=options,
            sort=sort,
        )

        response = self._post(
            document=json_query,
        )

        return response

    def vector_find(
        self,
        vector,
        *,
        limit,
        filter=None,
        fields=None,
        include_similarity=True,
    ):
        # Must pass a limit
        if not limit:
            return ValueError("Must pass a limit")

        # Pre-process the included arguments
        sort, projection = self._pre_process_find(
            vector,
            fields=fields,
        )

        # Call the underlying find() method to search
        raw_find_result = self.find(
            filter=filter,
            projection=projection,
            sort=sort,
            options={
                "limit": limit,
                "includeSimilarity": include_similarity,
            },
        )

        # Post-process the return
        find_result = self._post_process_find(
            raw_find_result,
            include_similarity=include_similarity,
        )

        return find_result

    @staticmethod
    def paginate(*, method, options, **kwargs):
        response0 = method(options=options, **kwargs)
        next_page_state = response0["data"]["nextPageState"]
        options0 = options
        for document in response0["data"]["documents"]:
            yield document
        while next_page_state is not None:
            options1 = {**options0, **{"pagingState": next_page_state}}
            response1 = method(options=options1, **kwargs)
            for document in response1["data"]["documents"]:
                yield document
            next_page_state = response1["data"]["nextPageState"]

    def paginated_find(self, filter=None, projection=None, sort=None, options=None):
        return self.paginate(
            method=self.find,
            filter=filter,
            projection=projection,
            sort=sort,
            options=options,
        )

    def pop(self, filter, update, options):
        json_query = make_payload(
            top_level="findOneAndUpdate", filter=filter, update=update, options=options
        )

        response = self._request(
            method=http_methods.POST,
            path=self.base_path,
            json_data=json_query,
        )

        return response

    def push(self, filter, update, options):
        json_query = make_payload(
            top_level="findOneAndUpdate", filter=filter, update=update, options=options
        )

        response = self._request(
            method=http_methods.POST,
            path=self.base_path,
            json_data=json_query,
        )

        return response

    def find_one_and_replace(
        self, sort={}, filter=None, replacement=None, options=None
    ):
        json_query = make_payload(
            top_level="findOneAndReplace",
            filter=filter,
            replacement=replacement,
            options=options,
            sort=sort,
        )

        response = self._request(
            method=http_methods.POST, path=f"{self.base_path}", json_data=json_query
        )

        return response

    def vector_find_one_and_replace(
        self,
        vector,
        replacement,
        *,
        filter=None,
        fields=None,
    ):
        # Pre-process the included arguments
        sort, _ = self._pre_process_find(
            vector,
            fields=fields,
        )

        # Call the underlying find() method to search
        raw_find_result = self.find_one_and_replace(
            replacement=replacement,
            filter=filter,
            sort=sort,
        )

        # Post-process the return
        find_result = self._post_process_find(
            raw_find_result,
            include_similarity=False,
            _key="document",
        )

        return find_result

    def find_one_and_update(self, sort={}, update=None, filter=None, options=None):
        json_query = make_payload(
            top_level="findOneAndUpdate",
            filter=filter,
            update=update,
            options=options,
            sort=sort,
        )

        response = self._request(
            method=http_methods.POST,
            path=f"{self.base_path}",
            json_data=json_query,
        )

        return response

    def vector_find_one_and_update(
        self,
        vector,
        update,
        *,
        filter=None,
        fields=None,
    ):
        # Pre-process the included arguments
        sort, _ = self._pre_process_find(
            vector,
            fields=fields,
        )

        # Call the underlying find() method to search
        raw_find_result = self.find_one_and_update(
            update=update,
            filter=filter,
            sort=sort,
        )

        # Post-process the return
        find_result = self._post_process_find(
            raw_find_result,
            include_similarity=False,
            _key="document",
        )

        return find_result

    def find_one(self, filter={}, projection={}, sort={}, options={}):
        json_query = make_payload(
            top_level="findOne",
            filter=filter,
            projection=projection,
            options=options,
            sort=sort,
        )

        response = self._post(
            document=json_query,
        )

        return response

    def vector_find_one(
        self,
        vector,
        *,
        filter=None,
        fields=None,
        include_similarity=True,
    ):
        # Pre-process the included arguments
        sort, projection = self._pre_process_find(
            vector,
            fields=fields,
        )

        # Call the underlying find() method to search
        raw_find_result = self.find_one(
            filter=filter,
            projection=projection,
            sort=sort,
            options={"includeSimilarity": include_similarity},
        )

        # Post-process the return
        find_result = self._post_process_find(
            raw_find_result,
            include_similarity=include_similarity,
            _key="document",
        )

        return find_result

    def insert_one(self, document):
        json_query = make_payload(top_level="insertOne", document=document)

        response = self._request(
            method=http_methods.POST, path=self.base_path, json_data=json_query
        )

        return response

    def insert_many(self, documents, options=None, partial_failures_allowed=False):
        json_query = make_payload(
            top_level="insertMany", documents=documents, options=options
        )

        response = self._request(
            method=http_methods.POST,
            path=f"{self.base_path}",
            json_data=json_query,
            skip_error_check=partial_failures_allowed,
        )

        return response

    def update_one(self, filter, update):
        json_query = make_payload(top_level="updateOne", filter=filter, update=update)

        response = self._request(
            method=http_methods.POST,
            path=f"{self.base_path}",
            json_data=json_query,
        )

        return response

    def replace(self, path, document):
        return self._put(path=path, document=document)

    def delete(self, id):
        json_query = {
            "deleteOne": {
                "filter": {"_id": id},
            }
        }

        response = self._request(
            method=http_methods.POST, path=f"{self.base_path}", json_data=json_query
        )

        return response

    def delete_subdocument(self, id, subdoc):
        json_query = {
            "findOneAndUpdate": {
                "filter": {"_id": id},
                "update": {"$unset": {subdoc: ""}},
            }
        }

        response = self._request(
            method=http_methods.POST, path=f"{self.base_path}", json_data=json_query
        )

        return response

    def upsert(self, document):
        """
        Emulate an upsert operation for a single document,
        whereby a document is inserted if its _id is new, or completely
        replaces and existing one if that _id is already saved in the collection.
        Returns: the _id of the inserted document.
        """
        # Attempt to insert the given document
        result = self.insert_one(document)

        # Check if we hit an error
        if (
            "errors" in result
            and "errorCode" in result["errors"][0]
            and result["errors"][0]["errorCode"] == "DOCUMENT_ALREADY_EXISTS"
        ):
            # Now we attempt to update
            result = self.find_one_and_replace(
                filter={"_id": document["_id"]},
                replacement=document,
            )
            upserted_id = result["data"]["document"]["_id"]
        else:
            upserted_id = result["status"]["insertedIds"][0]

        return upserted_id


class AstraDB:
    def __init__(
        self,
        token=None,
        api_endpoint=None,
        namespace=None,
    ):
        if token is None or api_endpoint is None:
            raise AssertionError("Must provide token and api_endpoint")

        if namespace is None:
            logger.info(
                f"ASTRA_DB_KEYSPACE is not set. Defaulting to '{DEFAULT_KEYSPACE_NAME}'"
            )
            namespace = DEFAULT_KEYSPACE_NAME

        # Store the initial parameters
        self.token = token
        (
            self.database_id,
            self.database_region,
            self.database_domain,
        ) = parse_endpoint_url(api_endpoint)

        # Set the Base URL for the API calls
        self.base_url = (
            f"https://{self.database_id}-{self.database_region}.{self.database_domain}"
        )
        self.base_path = f"{DEFAULT_BASE_PATH}/{namespace}"

        # Set the namespace parameter
        self.namespace = namespace

    def _request(self, *args, skip_error_check=False, **kwargs):
        response = make_request(
            *args,
            **kwargs,
            base_url=self.base_url,
            auth_header=DEFAULT_AUTH_HEADER,
            token=self.token,
        )

        responsebody = response.json()

        if not skip_error_check and "errors" in responsebody:
            raise ValueError(json.dumps(responsebody["errors"]))
        else:
            return responsebody

    def collection(self, collection_name):
        return AstraDBCollection(collection_name=collection_name, astra_db=self)

    def get_collections(self):
        response = self._request(
            method=http_methods.POST,
            path=self.base_path,
            json_data={"findCollections": {}},
        )

        return response

    def create_collection(
        self, collection_name, *, options=None, dimension=None, metric=""
    ):
        # Make sure we provide a collection name
        if not collection_name:
            raise ValueError("Must provide a collection name")

        # Initialize options if not passed
        if not options:
            options = {"vector": {}}
        elif "vector" not in options:
            options["vector"] = {}

        # Now check the remaining parameters - dimension
        if dimension:
            if "dimension" not in options["vector"]:
                options["vector"]["dimension"] = dimension
            else:
                raise ValueError(
                    "dimension parameter provided both in options and as function parameter."
                )

        # Check the metric parameter
        if metric:
            if "metric" not in options["vector"]:
                options["vector"]["metric"] = metric
            else:
                raise ValueError(
                    "metric parameter provided both in options as function parameter."
                )

        # Build the final json payload
        jsondata = {"name": collection_name, "options": options}

        # Make the request to the endpoitn
        self._request(
            method=http_methods.POST,
            path=f"{self.base_path}",
            json_data={"createCollection": jsondata},
        )

        # Get the instance object as the return of the call
        return AstraDBCollection(astra_db=self, collection_name=collection_name)

    def delete_collection(self, collection_name):
        # Make sure we provide a collection name
        if not collection_name:
            raise ValueError("Must provide a collection name")

        response = self._request(
            method=http_methods.POST,
            path=f"{self.base_path}",
            json_data={"deleteCollection": {"name": collection_name}},
        )

        return response
